/* Tabela CUSTOLOGIST: Custo de Logstica */
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela CUSTOLOGIST */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela CUSTOLOGIST,
             utilizando a tabela Geral de Logs de Operaes */


IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_GER_CUSTOLOGIST' AND
                 TYPE   = 'TR' 	               AND
	              U.UID  = O.UID                 AND
	              U.NAME = USER)
DROP TRIGGER LOG_GER_CUSTOLOGIST
GO


CREATE TRIGGER LOG_GER_CUSTOLOGIST
ON CUSTOLOGIST
FOR INSERT, UPDATE, DELETE
AS

   DECLARE  @V_USUARIO   VARCHAR(40);
   DECLARE  @V_TIPO      VARCHAR(1);
   DECLARE  @V_ITPESQ    VARCHAR(40);
   DECLARE  @V_DESC      VARCHAR(300);
   DECLARE  @V_DTOPER    DATETIME;


/*VARIVEIS DO CURSOR C_CURSOR*/

   DECLARE  @V_ID_COMANDO       VARCHAR(1);
   DECLARE  @V_OLD_CDFILIAL     VARCHAR(4);
   DECLARE  @V_OLD_CDFORNECED   VARCHAR(21);
   DECLARE  @V_OLD_CDPRODUTO    VARCHAR(10);
   DECLARE  @V_OLD_DTINIVIGCLOG DATETIME;
   DECLARE  @V_OLD_VRPECUSTOLOG DOUBLE PRECISION;
   DECLARE  @V_OLD_VRVRCUSTOLOG DOUBLE PRECISION;
   DECLARE  @V_OLD_VRPESEGURLOG DOUBLE PRECISION;

   DECLARE  @V_NEW_CDFILIAL     VARCHAR(4);
   DECLARE  @V_NEW_CDFORNECED   VARCHAR(21);
   DECLARE  @V_NEW_CDPRODUTO    VARCHAR(10);
   DECLARE  @V_NEW_DTINIVIGCLOG DATETIME;
   DECLARE  @V_NEW_VRPECUSTOLOG DOUBLE PRECISION;
   DECLARE  @V_NEW_VRVRCUSTOLOG DOUBLE PRECISION;
   DECLARE  @V_NEW_VRPESEGURLOG DOUBLE PRECISION;

BEGIN
   SET @V_TIPO = '0';
   SET @V_DESC = '';
   EXEC NMUSUARIO @V_USUARIO;
   SET @V_USUARIO = ISNULL(@V_USUARIO, '');

   DECLARE C_CURSOR CURSOR FOR
   SELECT
     'D' IDCOMANDO,
         CDFILIAL,
         CDFORNECED,
         CDPRODUTO,
         DTINIVIGCLOG,
         VRPECUSTOLOG,
         VRVRCUSTOLOG,
         VRPESEGURLOG,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL
   FROM
     DELETED
   WHERE
     CDFILIAL+CDFORNECED+CDPRODUTO+CONVERT(VARCHAR, DTINIVIGCLOG) NOT IN (SELECT
                                                                          CDFILIAL+CDFORNECED+CDPRODUTO+
                                                                          CONVERT(VARCHAR, DTINIVIGCLOG)
                                                                          FROM INSERTED)
   UNION ALL

   SELECT
     'U' IDCOMANDO,
         DELETED.CDFILIAL,
         DELETED.CDFORNECED,
         DELETED.CDPRODUTO,
         DELETED.DTINIVIGCLOG,
         DELETED.VRPECUSTOLOG,
         DELETED.VRVRCUSTOLOG,
         DELETED.VRPESEGURLOG,
         INSERTED.CDFILIAL,
         INSERTED.CDFORNECED,
         INSERTED.CDPRODUTO,
         INSERTED.DTINIVIGCLOG,
         INSERTED.VRPECUSTOLOG,
         INSERTED.VRVRCUSTOLOG,
         INSERTED.VRPESEGURLOG

   FROM
      INSERTED,
      DELETED
   WHERE
      INSERTED.CDFILIAL     = DELETED.CDFILIAL    AND
      INSERTED.CDFORNECED   = DELETED.CDFORNECED  AND
      INSERTED.CDPRODUTO    = DELETED.CDPRODUTO   AND
      INSERTED.DTINIVIGCLOG = DELETED.DTINIVIGCLOG
   UNION ALL

   SELECT
      'I' IDCOMANDO,
          NULL,
          NULL,
          NULL,
          NULL,
          NULL,
          NULL,
          NULL,
          CDFILIAL,
          CDFORNECED,
          CDPRODUTO,
          DTINIVIGCLOG,
          VRPECUSTOLOG,
          VRVRCUSTOLOG,
          VRPESEGURLOG
   FROM
      INSERTED
   WHERE
      CDFILIAL+CDFORNECED+CDPRODUTO+CONVERT(VARCHAR, DTINIVIGCLOG) NOT IN (SELECT
                                                                           CDFILIAL+CDFORNECED+CDPRODUTO+
                                                                           CONVERT(VARCHAR, DTINIVIGCLOG)
                                                                           FROM DELETED);

   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                 @V_OLD_CDFILIAL,
                                 @V_OLD_CDFORNECED,
                                 @V_OLD_CDPRODUTO,
                                 @V_OLD_DTINIVIGCLOG,
                                 @V_OLD_VRPECUSTOLOG,
                                 @V_OLD_VRVRCUSTOLOG,
                                 @V_OLD_VRPESEGURLOG,

                                 @V_NEW_CDFILIAL,
                                 @V_NEW_CDFORNECED,
                                 @V_NEW_CDPRODUTO,
                                 @V_NEW_DTINIVIGCLOG,
                                 @V_NEW_VRPECUSTOLOG,
                                 @V_NEW_VRVRCUSTOLOG,
                                 @V_NEW_VRPESEGURLOG;


   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF @V_ID_COMANDO = 'D' BEGIN
         IF @V_OLD_CDFILIAL IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Filial=' + @V_OLD_CDFILIAL;
         END;

         IF @V_OLD_CDFORNECED IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Fornecedor=' + @V_OLD_CDFORNECED;
         END;

         IF @V_OLD_CDPRODUTO IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Produto=' + @V_OLD_CDPRODUTO;
         END;

         IF @V_OLD_DTINIVIGCLOG IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' Vigencia=' + CONVERT(VARCHAR, @V_OLD_DTINIVIGCLOG);
         END;

         IF @V_OLD_VRPECUSTOLOG IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' PercentoLog=' + CONVERT(VARCHAR,@V_OLD_VRPECUSTOLOG);
         END;

         IF @V_OLD_VRVRCUSTOLOG IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' ValorLog=' + CONVERT(VARCHAR,@V_OLD_VRVRCUSTOLOG);
         END;

         IF @V_OLD_VRPESEGURLOG IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + ' PercentoSeg=' + CONVERT(VARCHAR,@V_OLD_VRPESEGURLOG);
         END;

         SET @V_TIPO = '3';
         SET @V_ITPESQ = @V_OLD_CDFILIAL+@V_OLD_CDFORNECED+@V_OLD_CDPRODUTO+CONVERT(VARCHAR, @V_OLD_DTINIVIGCLOG);
      END
      ELSE BEGIN
         IF @V_ID_COMANDO= 'U' /*UPDATING*/ BEGIN
            IF (ISNULL(@V_OLD_VRPECUSTOLOG,0) <> ISNULL(@V_NEW_VRPECUSTOLOG,0) ) OR
               (ISNULL(@V_OLD_VRVRCUSTOLOG,0) <> ISNULL(@V_NEW_VRVRCUSTOLOG,0) ) OR
               (ISNULL(@V_OLD_VRPESEGURLOG,0) <> ISNULL(@V_NEW_VRPESEGURLOG,0) ) BEGIN

               SET @V_TIPO   = '2';
               SET @V_ITPESQ = @V_OLD_CDFILIAL+@V_OLD_CDFORNECED+@V_OLD_CDPRODUTO+CONVERT(VARCHAR,@V_OLD_DTINIVIGCLOG);
            END;
         END
         ELSE BEGIN
            SET @V_TIPO   = '1';
            SET @V_ITPESQ = @V_NEW_CDFILIAL+@V_NEW_CDFORNECED+@V_NEW_CDPRODUTO+CONVERT(VARCHAR,@V_NEW_DTINIVIGCLOG);
         END;

         SET @V_DESC = @V_DESC + 'Filial='      + ISNULL(@V_OLD_CDFILIAL,     @V_NEW_CDFILIAL);
         SET @V_DESC = @V_DESC + ' Fornecedor=' + ISNULL(@V_OLD_CDFORNECED,   @V_NEW_CDFORNECED);
         SET @V_DESC = @V_DESC + ' Produto='    + ISNULL(@V_OLD_CDPRODUTO,    @V_NEW_CDPRODUTO);
         SET @V_DESC = @V_DESC + ' Vigencia='   + ISNULL(CONVERT(VARCHAR, @V_OLD_DTINIVIGCLOG),
                                                         CONVERT(VARCHAR, @V_NEW_DTINIVIGCLOG));

         IF (ISNULL(@V_OLD_VRPECUSTOLOG,0) <> ISNULL(@V_NEW_VRPECUSTOLOG,0) ) BEGIN
            SET @V_DESC = @V_DESC + ' PercentoLog=' + CONVERT(VARCHAR,@V_NEW_VRPECUSTOLOG);
         END;

         IF (ISNULL(@V_OLD_VRVRCUSTOLOG,0) <> ISNULL(@V_NEW_VRVRCUSTOLOG,0) ) BEGIN
            SET @V_DESC = @V_DESC + ' ValorLog=' + CONVERT(VARCHAR,@V_NEW_VRVRCUSTOLOG);
         END;

         IF (ISNULL(@V_OLD_VRPESEGURLOG,0) <> ISNULL(@V_NEW_VRPESEGURLOG,0) ) BEGIN
            SET @V_DESC = @V_DESC + ' PercentoSeg=' + CONVERT(VARCHAR,@V_NEW_VRPESEGURLOG);
         END;
      END;

      IF (@V_TIPO <> '0')  BEGIN
         IF NOT @V_DESC IS NULL BEGIN
            IF NOT @V_USUARIO IS NULL BEGIN
               SET @V_DTOPER = GETDATE();
               INSERT INTO LOGOPERA
                  (NMTABELA,   DTOPERACAO, NMRESPOPER,
                   IDTIPOOPER, CDPESQITEM, DSOPERACAO)
               VALUES
                  ('CUSTOLOGIST',  @V_DTOPER, @V_USUARIO,
                   @V_TIPO,        @V_ITPESQ, @V_DESC);
            END;
         END;
      END;

      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_OLD_CDFILIAL,
                                    @V_OLD_CDFORNECED,
                                    @V_OLD_CDPRODUTO,
                                    @V_OLD_DTINIVIGCLOG,
                                    @V_OLD_VRPECUSTOLOG,
                                    @V_OLD_VRVRCUSTOLOG,
                                    @V_OLD_VRPESEGURLOG,

                                    @V_NEW_CDFILIAL,
                                    @V_NEW_CDFORNECED,
                                    @V_NEW_CDPRODUTO,
                                    @V_NEW_DTINIVIGCLOG,
                                    @V_NEW_VRPECUSTOLOG,
                                    @V_NEW_VRVRCUSTOLOG,
                                    @V_NEW_VRPESEGURLOG;
   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'LOG_GER_CUSTOLOGIST','4.11.105';
GO



